 /*
  PePo (c)by Axel Muhr - www.geekdot.com
  v1.0 - 20.10.2015 - initial relase

  A very, very simple Peek/Poke Tool to have a quick look into
  a CPUs memory which is connected to an INMOS link interface
  and uses the INMOS protocol.
  This should work with all B004 compatible interfaces, even
  with the NumberSmasher i860 (which hasn't a Transputer at all).

  I was too lazy to define a switch for setting the port address.
  It's either 0x150 or read from the enviornment variable TRANSPUTER
  which by INMOS standards reads e.g. "#200" for 0x200.
  The conversion is lame and needs more love, strtol for example.

*/
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <time.h>

#define BASE 0x150

#define PORT_RD_DATA 0 // (BASE+0)
#define PORT_WR_DATA 1 // (BASE+1)
#define PORT_RD_STAT 2 // (BASE+2)
#define PORT_WR_STAT 3 // (BASE+3)
#define PORT_SS_R    0x10 //(BASE+0x10)

#define TIMEOUT	2
#define TRUE 1
#define FALSE 0

int C012_timeout = FALSE;
int PortBase = 0x150;

/* ------------------------------------------------------------------------ */
unsigned char RecvByte( void ) {
/* ------------------------------------------------------------------------ */

    while (( inportb( PortBase + PORT_RD_STAT ) & 1 ) == 0 );
    return inportb( PortBase + PORT_RD_DATA );
}

/* ------------------------------------------------------------------------ */
void SendByte( unsigned char b ) {
/* ------------------------------------------------------------------------ */

    while (( inportb( PortBase + PORT_WR_STAT ) & 1 ) == 0 );
    outportb( PortBase + PORT_WR_DATA, b );
}

/* ------------------------------------------------------------------------ */
unsigned long int Peek( unsigned long int a ) {
/* ------------------------------------------------------------------------ */

    unsigned long int d;
    unsigned char *p;

    SendByte( 1 );

    p = (unsigned char *)&a;
    SendByte( p[ 0 ] );
    SendByte( p[ 1 ] );
    SendByte( p[ 2 ] );
    SendByte( p[ 3 ] );

    p = (unsigned char *)&d;
    p[ 0 ] = RecvByte();
    p[ 1 ] = RecvByte();
    p[ 2 ] = RecvByte();
    p[ 3 ] = RecvByte();

    return d;
}

/* ------------------------------------------------------------------------ */
void Poke( unsigned long int a, unsigned long int d ) {
/* ------------------------------------------------------------------------ */

    unsigned char *p;

    SendByte( 0 );

    p = (unsigned char *)&a;
    SendByte( p[ 0 ] );
    SendByte( p[ 1 ] );
    SendByte( p[ 2 ] );
    SendByte( p[ 3 ] );

    p = (unsigned char *)&d;
    SendByte( p[ 0 ] );
    SendByte( p[ 1 ] );
    SendByte( p[ 2 ] );
    SendByte( p[ 3 ] );
}

/* ------------------------------------------------------------------------ */
void Reset( void ) {
/* ------------------------------------------------------------------------ */

    outportb( PortBase + PORT_SS_R, 1 );
    delay( 100 );
    outportb( PortBase + PORT_SS_R, 0 );
    delay(   1 );
}

void usage() {
	printf( "PePo command address option (in Hex!), e.g.\n\tpeek address [steps]\n\tpoke address data\n\treset\n");
	exit(1);
}

/* ======================================================================== */
int main( int argc, char* argv[] ) {
/* ======================================================================== */

unsigned long lnumber1, lnumber2, lnumber3, pR;
unsigned char uc1, uc2, uc3, uc4;
char *trans_env;
char* trans_env_chopped;

int step = 0;
unsigned long int i;

    if ( argc <= 1 ) {
	usage();
    }

	trans_env = getenv("TRANSPUTER");
	if (strlen(trans_env) != 0) {
	  trans_env_chopped = trans_env + 1; // mv pointer,get rid of the "#"
	  PortBase = atoi(trans_env_chopped);
	  printf("Base is %i", PortBase);
	}


		if (strcmp(argv[1],"peek") == 0) {
			if (atoi(argv[2]) % 4 != 0) {
				printf("Address needs to be devideable by 4!\n");
				exit(1);
			}

			if (argc > 2) {
				step = atoi(argv[3]);
				printf("Dumping %i adresses:\n\n", step);
			}
			lnumber1 = strtoul(argv[2], NULL, 16);
			for(i=0; i<=step; i++) {
			 pR = Peek(4 * i + lnumber1);
			 uc1 = (unsigned char)(pR >> 24);
			 uc2 = (unsigned char)(pR >> 16);
			 uc3 = (unsigned char)(pR >> 8);
			 uc4 = (unsigned char)(pR >> 0);

			 printf("%lx : %02X %02X %02X %02X\n",4*i+lnumber1,uc1, uc2, uc3, uc4);
			}
		} else if (strcmp(argv[1],"poke") == 0) {
			if (argc < 3) {
				printf("No data given!\n");
				exit(1);
			}
			fflush( stdout );
			lnumber2 = strtoul(argv[2], NULL, 16);
			lnumber3 = strtoul(argv[3], NULL, 16);
			printf("Poke %lu to address %lu\n",lnumber3, lnumber2);
			Poke(lnumber2, lnumber3);
		} else if (strcmp(argv[1],"reset") == 0){
			printf("Resetting...");
			Reset();
			printf("done\n");
		} else {
			printf("Unknown command!\n");
			usage();
		}
		return(0);
}

